{comb2.dsp - FIR comb filter implemented recursively}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Eq.(8.2.8) of Introduction to Signal Processing.
 I/O equation: y(n) = a0 x(n) + a1 x(n-D) + a2 x(n-2D) + a3 x(n-3D)
 Sample processing algorithm:
       for each x do:
           s1 = tap(4*D, w, p, D)                       get tap-D
           s4 = tap(4*D, w, p, 4*D)                     get tap-4D
           *p = s0 = x + a * s1
           y =  s0 - a4 * s4                            filter output
           cdelay(4*D, w, &p)                           update delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const a0 = 0x7fff;                {a0 = 0.9999 = approximate unity}
.const a1 = 0x4000;                {a1 = 0.50}
.const a4 = 0x0800;                {a4 = a1^4 = 0.50^4 = 0.0625}

.const D = 1600;                   {TD = D/fs = 1600/8000 = 0.20 sec}
.var/dm/circ w[4*D + 1];           {delay-line buffer, max delay = 4*D}

i2 = ^w;  L2 = %w;                 {delay-line buffer pointer and length}

zero(i2, m2, L2);                  {clear delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   mr = 0;
   mr1 = mx1;                      {mr = x = input}

   tap(i2, m2, D, my0);            {my0 = s1 = D-th tap}
   mx0 = a1;
   mr = mr + mx0 * my0 (rnd);      {mr = s0 =  x + a1 * s1}
   if mv sat mr;                   {saturate mr if overflow}
   sr0 = mr1;                      {sr0 = s0}

   mx0 = a4;
   tap(i2, m2, 4*D, my0);          {my0 = s4 = 4D-th tap}
   mr = mr - mx0 * my0 (rnd);      {mr = y = s0 - a4 * s4}
   if mv sat mr;                   {saturate mr if overflow}

   tapin(i2, m2, sr0);             {put input from sr0 into tap-0}
   cdelay(i2, m2);                 {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = mr1;           {left output sample}
   dm(tx_buf + 2) = mr1;           {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
