{delay.dsp - plain delay by TD sec}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on cdelay.c of Introduction to Signal Processing, p.177.
 I/O equation: y(n) = x(n-d), range of delay d=1, 2, ..., D.
 Sample processing algorithm:
       for each input x do:
           y = tap(D, w, p, d)            get d-th tap
           *p = x                         put input x into tap-0
           cdelay(D, w, &p)               update delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const D = 6000;            {max delay = TD = D/fs = 6000/8000 = 3/4 sec}
.const d = 2000;            {d-th tap, d = 1,2,...,D}
.var/dm/circ w[D+1];        {define delay-line buffer}

i2 = ^w;  L2 = %w;          {buffer pointer and length}

zero(i2, m2, L2);           {clear delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   tap(i2, m2, d, my1);            {d-th tap output into my1}
   tapin(i2, m2, mx1);             {input from mx1 into tap-0}
   cdelay(i2, m2);                 {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = my1;           {left output sample}
   dm(tx_buf + 2) = my1;           {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
