% firlp.m - lowpass FIR design
% Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996
%
% the filter coeff file, lp.dec, in decimal format must be postprocessed
% by dec2hex to convert it to the 1.15 format hex file lp.hex, that is,
% dec2hex 1 15 < lp.dec > lp.hex
%
% to run it on the EZ-KIT Lite, 
% copy lp.hex into fir.hex, recompile and load fir.dsp
% (you may also need to edit the designed filter order M into fir.dsp)

fs = 8000;                  % 8 kHz rate

Apass = 0.1;                % 0.1 dB
Astop = 40;                 % 40 dB, stopband ripple: delta = 0.01

fc = input('cutoff frequency fc (in Hz) =  ');    % e.g., fc = 200 Hz
Df = input('transition width Df (in Hz) =  ');    % e.g., Df = 100 Hz

fpass = fc - Df/2;
fstop = fc + Df/2;

h = klh(1, fs, fpass, fstop, Apass, Astop)';            % Kaiser design

save lp.dec h /ascii                      % save in decimal format

[N, N1] = size(h); M = N-1                % filter order M

NF = 400;                                 % number of frequencies
w = (0:NF-1) * pi / NF;                   % NF freqs over positive Nyquist
H = 20 * log10(abs(dtft(h', w)))';        % magnitude response in dB

save lpmag.dat H /ascii
