{phaser.dsp - phasing effect with a variable notch filter}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Section 8.2.2 and Eq.(8.2.22) if Introd. to Signal Processing.
 Design width Df = 80 Hz, b0 = 1 / (1 + tan(Dw/2)) = 0.9695.
 Notch frequency varies sinusoidally from 200 Hz to 800 Hz:

       f0 = 500 + 300 * sin(2*pi*fsweep*t)              (in Hz)
       w0 = 0.125*pi + 0.075*pi * sin(2*pi*fsweep*t)    (in rads/sample)

 Sweep frequency fsweep = 10 Hz. Because w0 varies between
       0.20*pi < w0 < 0.05*pi
 we can use the approximation cos(w0) = 1 - w0^2 / 2.
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.var/dm/circ w[3];                 {y-delay-line buffer in DM}
.var/dm/circ v[3];                 {x-delay-line buffer in DM}
.var/pm/circ a[3], b[3];           {concatenated coeffs in PM}

.const ea = 1;                     {scaling exponent for a}
.const eb = 1;                     {scaling exponent for b}
.const ex = 1;                     {input scaling exponent}

i2 = ^w; L2 = %w;                  {y-delay-line pointer}
i3 = ^v; L3 = %v;                  {x-delay-line pointer}
i4 = ^a; L4 = 6;                   {double-length a,b coefficients}

zero(i2, m2, L2);                  {clear y-delay line}
zero(i3, m3, L3);                  {clear x-delay line}

.const Ds = 4000;                  {wavetable's min freq: fs/Ds = 2 Hz}
.var/dm/circ s[Ds];                {cosinusoidal wavwetable}

i6 = ^s; L6 = %s;                  {points to sweep frequency}

.const csweep = 5;                 {fsweep = 10 Hz}

.const w1 = 0x3244;                {w1 = 0.125*pi => f1 = 500 Hz}
.const w2 = 0x1e28;                {w2 = 0.075*pi => f2 = 300 Hz}

.const b0  = 0x3e0c;               {b0  = 0.9695, scaled by 2}
.const b11 = 0x83e7;               {b11 = -2*b0 = -1.9390, scaled by 2}
.const a2  = 0x3c19;               {a2  = 2*b0-1 = 0.9390, scaled by 2}
.const a0  = 0x4000;               {a0  = 1, scaled by 2}

.init s: <sinetbl.hex>;            {load wavetable}
.init a: a0, 0, a2;                {initial denominator coefficients}
.init b: b0, 0, b0;                {initial numerator coefficients}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);                  {left input sample}
   mx1 = dm(rx_buf + 2);                  {right input sample}

{--- sample processing algorithm -----------------------------------------}

   ax0 = w1;
   wavgen(i6, m6, w2, csweep, ay0);       {ay0 = w2 * sine(sweep*n)}
   ar = ax0 + ay0;                        {ar = w0 = w1 + w2*sin(wsweep*n)}

   mr= 0;                                 {calculate cos(w0)}
   mr1 = 0x7fff;                          {mr = 1}
   sr = ashift ar by -1 (hi);             {sr1 = w0 / 2}
   my0 = ar;                              {my0 = w0}
   mr = mr - sr1 * my0 (rnd);             {mr1 = cos(w0) = 1 - w0^2 / 2}

   my0 = b11;                             {current a1 and b1 coefficients}
   mr = ar * my0 (rnd);                   {mr1 = -2 * b0 * cos(w0)}

   m5 = 1; 
   modify(i4, m5);                        {start from a0, point to a1}
   pm(i4, m5) = sr1;                      {save current a1, point to a2}

   m5 = 2; 
   modify(i4, m5);                        {point to b1}
   pm(i4, m5) = sr1;                      {save b1, point back to a0}

   sr1 = mx1;

  {jump nofilter;}                        {uncomment to bypass filter}
              
   sr = ashift sr1 by -ex (hi);                         {pre-scale x down}
                                                        {compute output y}
   cdir(2, i4, m4, i2, m2, i3, m3, ea, eb, sr1);        {input from sr1}
                                                        {output in ar}
   sr = ashift ar by ex (hi);                           {post-scale y up}
   sr = ashift sr1 by 1 (hi);                   {make it a little louder}

{--- write samples to codec ---------------------------------------------}

nofilter:

   dm(tx_buf + 1) = sr1;                   {left output sample}
   dm(tx_buf + 2) = sr1;                   {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
