{multitap.dsp - multitap delay line with feedforward and feedback}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Fig. 8.2.29 of Introduction to Signal Processing.
 The sample processing algorithm is:
     for each x do:
          s1 = tap(D1+D2, w, p, D1)              D1-th tap output
          s2 = tap(D1+D2, w, p, D1+D2)           (D1+D2)-th tap output
          y = b0 * x + b1 * s1 + b2 * s2         filter output
          *p = s0 = x + a1 * s1 + a2 * s2        put input s0 into delay
          cdelay(D1+D2, w, &p)                   update delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const a1 = 0x199a;                {a1 = 0.20}
.const a2 = 0x4000;                {a2 = 0.50}

.const b0 = 0x7fff;                {b0 = 0.9999, that is, 1}
.const b1 = 0x6666;                {b1 = 0.80}
.const b2 = 0x4ccd;                {b2 = 0.60}

.const D1 = 3000;                  {D1/fs = 3000/8000 = 0.375 sec}
.const D2 = 4500;                  {D1+D2 = 7500, 7500/8000 = 0.9375 sec}

.var/dm/circ w[D1+D2+1];           {delay-line buffer}

i2 = ^w; L2 = %w;                  {delay-line buffer pointer and length}

zero(i2, m2, L2);                  {clear delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   tap(i2, m2, D1, sr0);           {tap s1 into sr0}
   tap(i2, m2, D1+D2, sr1);        {tap s2 into sr1}

   my0 = b0;
   mr = mx1 * my0 (ss);            {mr = b0 * x}
   my0 = b1;
   mr = mr + sr0 * my0 (ss);       {mr = b0 * x + b1 * s1}
   my0 = b2;
   mr = mr + sr1 * my0 (rnd);      {mr = y = b0 * x + b1 * s1 + b2 * s2}
   if mv sat mr;

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = mr1;           {left output sample}
   dm(tx_buf + 2) = mr1;           {right output sample}

{--- sample processing algorithm (cont'd) --------------------------------}

   mr = 0;
   mr1 = mx1;                      {mr = x}
   my0 = a1;
   mr = mr + sr0 * my0 (ss);       {mr = x + a1 * s1}
   my0 = a2;
   mr = mr + sr1 * my0 (rnd);      {mr = s0 = x + a1 * s1 + a2 * s2}
   if mv sat mr;

   tapin(i2, m2, mr1);             {input s0 into delay}
   cdelay(i2, m2);                 {update delay}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
