{notch3.dsp - double-notch filter at 800 Hz and 2400 Hz - canonical form}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Designed with parmeq.m of Appendix D, with the MATLAB commands:
       GB = 1/sqrt(2); f1 = 800; f3 = 2400; fs = 8000; Df = 50;
       [b1, a1, beta1] = parmeq(1, 0, GB, 2*pi*f1/fs, 2*pi*Df/fs);
       [b3, a3, beta3] = parmeq(1, 0, GB, 2*pi*f3/fs, 2*pi*Df/fs);
       a13 = conv(a1, a3); b13 = conv(b1, b3);
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const A0 = 0x4000;                {A0 = 0.5 = input scale factor}

.const M = 4;                      {filter order}
.var/dm/circ w[M+1];               {circular delay-line buffer in DM}
.var/pm/circ a[M+1], b[M+1];       {concatenated coeffs in PM}

.const ea = 0;                     {scaling exponent for a}
.const eb = 0;                     {scaling exponent for b}
.const ex = 3;                     {input scaling exponent}

i2 = ^w; L2 = %w;                  {delay-line buffer pointer and length}
i4 = ^a; L4 = 2*(M+1);             {double-length a,b coefficients}

zero(i2, m2, L2);                  {clear delay line}

.const Ds = 800;                   {wavetable's min freq: fs/Ds = 10 Hz}
.var/dm/circ s[Ds];                {square wavetable}

i6 = ^s; L6 = %s;                  {wavetable pointer and length}

.const c = 80;                     {square wave frequency f1 = c * fs/Ds}
.const A = 0x1000;                 {A = A0/4 = square wave amplitude}

.init s: <squartbl.hex>;           {load wavetable}
.init a: <a13.hex>;                {denominator coefficients}
.init b: <b13.hex>;                {numerator coefficients}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);                  {left input sample}
   mx1 = dm(rx_buf + 2);                  {right input sample}

{--- sample processing algorithm -----------------------------------------}

   wavgen(i6, m6, A, c, mr1);             {mr1 = A * square wave}

   my1 = A0;
   mr = mr + mx1 * my1 (rnd);             {mr = x = A0 * mike + A * square}
   if mv sat mr;

  {sr1 = mr1; jump nofilter;}             {uncomment to bypass filter}
              
   sr = ashift mr1 by -ex (hi);                  {pre-scale x down}
                                                 {compute output y}
   ccan(M, i4, m4, i2, m2, ea, eb, sr1);         {input from sr1}
                                                 {output in sr1}
   sr = ashift sr1 by ex (hi);                   {post-scale y up}

{--- write samples to codec ---------------------------------------------}

nofilter:

   dm(tx_buf + 1) = sr1;                   {left output sample}
   dm(tx_buf + 2) = sr1;                   {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
