{plain2.dsp - plain reverb - IIR comb filter - in stereo}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on plain.c of Introduction to Signal Processing.
 Sample processing algorithm from Eq.(8.2.14) and Fig.8.2.6:
       for each x do :
           sD = tap(D, w, p, D)                  get D-th tap
           y = x + a * sD                        filter output
           *p = y                                put y into tap-0
           cdelay(D, w, &p)                      update delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const aL = 0x4000;                {left feedback coefficient aL = 0.50}
.const DL = 3000;                  {left TD = D/fs = 3/8 = 0.375 sec}

.const aR = 0x4000;                {right feedback coefficient aR = 0.50}
.const DR = 3000;                  {right TD = D/fs = 3/8 = 0.375 sec}

.var/dm/circ wL[DL+1];             {left delay-line buffer}
.var/dm/circ wR[DR+1];             {right delay-line buffer}

i2 = ^wL; L2 = %wL;                {left delay buffer pointer and length}
i3 = ^wR; L3 = %wR;                {right delay buffer pointer and length}

zero(i2, m2, L2);                  {clear left delay line}
zero(i3, m3, L3);                  {clear right delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm for left channel ------------------------}

   mr = 0;
   mr1 = ax1;                      {mr = x = left input}

   my0 = aL;
   tap(i2, m2, DL, mx0);           {mx0 = sD = D-th tap}
   mr = mr + mx0 * my0 (rnd);      {mr = y = x + a * sD = left output}
   
   tapin(i2, m2, mr1);             {put y in tap-0}
   cdelay(i2, m2);                 {update left delay}

   dm(tx_buf + 1) = mr1;           {left output sample}

{--- sample processing algorithm for right channel -----------------------}

   mr = 0;
   mr1 = mx1;                      {mr = x = right input}

   my0 = aR;
   tap(i3, m3, DR, mx0);           {mx0 = sD = D-th tap}
   mr = mr + mx0 * my0 (rnd);      {mr = y = x + a * sD = right output}
   
   tapin(i3, m3, mr1);             {put y in tap-0}
   cdelay(i3, m3);                 {update right delay}

   dm(tx_buf + 2) = mr1;           {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
