{dnsamp2.dsp - aliasing of a sinusoid by decimation}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const M = 10;                     {downsampling ratio; fs_low = fs/M}
.var/dm/circ s[M];                 {low-rate sampling pulse train}

.const D = 2000;                   {basic frequency increment fs/D = 4 Hz}
.var/dm/circ sine[D];              {sinusoidal wavetable, from sinetbl.hex}

.const FS = 8000;                  {fast sampling rate in samples/sec}
.const c = 250;                    {f = c * fs / D = c * 4 = 1000 Hz}
.const T = 4;                      {desired total duration in seconds}
.var/dm N;                         {N = fs * T = no. of samples in T sec}

ax1 = T * FS;                      {N = T * FS = duration in samples}
dm(N) = ax1;                       {save N in DM}

i5 = ^s; L5 = %s; m5 = 1;          {low-rate sampling pulse train}
i6 = ^sine; L6 = %sine; m6 = c;    {sine wavetable with increment c}

zero(i5, m5, L5);                  {initialize pulse train s(n) to zero}
ax0 = 0x7fff;                      {set s[0] = 1}
dm(s) = ax0;                       {s = [1,0,0,...,0] = one and M-1 zeros}

.init sine: <sinetbl.hex>;         {load sinusoidal wavetable}
                                   {from file sinetbl.hex}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- sample processing algorithm -----------------------------------------}

   ay0 = dm(N);
   ar = ay0 - 1;                   {decrement N until zero}
   dm(N) = ar;
   if eq jump stop;                {do not stop until N iterations}

   mr1 = dm(i6, m6);               {mr1 = sinusoidal input sin(2*pi*f*t)}

   ar = dm(i5, m5);                {get sampling pulse}
   ar = pass ar;                   {forces updating of AZ flag}
   if ne jump output;              {if AR != 0, write output}

   mr1 = 0;                        {else, output is zero}

   output:

#if 0                                            {optional amplify}
   sr = ashift mr1 by 1 (hi);  mr1 = sr1;
#endif

   dm(tx_buf + 1) = mr1;                         {left output sample}
   dm(tx_buf + 2) = mr1;                         {right output sample}

   rti;

stop:

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
