{revdel.dsp - reverberating delay with direct path}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Problem 8.17 of Introduction to Signal Processing.
 Sample processing algorithm:
       for each input x do:
           sD = tap(D, w, p, D);          D-th tap output of delay
           y = sD + c * x;                c = direct path gain, y = output
           s0 = b * x + a * sD;           b = gain before delay
           *p = s0;                       put s0 into delay's tap-0
           cdelay(D, w, &p);              update delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const a = 0x4000;                 {a = 0.5}     {feedback gain}
.const b = 0x7fff;                 {b = 0.9999}  {gain before delay}
.const c = 0x0000;                 {c = 0}       {gain for direct sound}

.const D = 6000;                   {TD = D/fs = 6/8 = 0.75 sec}
.var/dm/circ w[D+1];               {delay-line buffer}

i2 = ^w;  L2 = %w;                 {buffer pointer and length}

zero(i2, m2, L2);                  {clear delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);                  {left input sample}
   mx1 = dm(rx_buf + 2);                  {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   tap(i2, m2, D, sr1);                   {sr1 = sD}
   mr1 = sr1;

   my1 = c;
   mr = mr + mx1 * my1 (rnd);             {mr = y = sD + c * x}
   if mv sat mr;
   ar = mr1;                              {save for output}

   my1 = b;
   mr = mx1 * my1 (ss);                   {mr = b * x}
   my1 = a;
   mr = mr + sr1 * my1 (rnd);             {mr = s0 = b * x + a * sD}
   if mv sat mr;

   tapin(i2, m2, mr1);                    {put s0 into delay-line input}
   cdelay(i2, m2);                        {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = ar;                   {left output sample}
   dm(tx_buf + 2) = ar;                   {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
