{reverb.dsp - Schroeder's reverberator}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Fig.8.2.18 of Introduction to Signal Processing.
 Sample processing algorithm is given by Eq.(8.2.31) and the code is
 taken from plain.dsp and allpass.dsp.}

{--- choose sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc85b; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const b1 = 0x7fff;         {b1 = 0.9999}
.const b2 = 0x7333;         {b2 = 0.90}
.const b3 = 0x6666;         {b3 = 0.80}
.const b4 = 0x599a;         {b4 = 0.70}

.const a1 = 0x70a4;         {a1 = 0.88}
.const a2 = 0x70a4;         {a2 = 0.88}
.const a3 = 0x70a4;         {a3 = 0.88}
.const a4 = 0x70a4;         {a4 = 0.88}
.const a5 = 0x70a4;         {a5 = 0.88}
.const a6 = 0x70a4;         {a6 = 0.88}

.const D1 = 1759;
.const D2 = 1949;
.const D3 = 2113;
.const D4 = 2293;
.const D5 = 307;
.const D6 = 313;

.var/dm/circ w1[D1 + 1];
.var/dm/circ w2[D2 + 1];
.var/dm/circ w3[D3 + 1];
.var/dm/circ w4[D4 + 1];
.var/dm/circ w5[D5 + 1];
.var/dm/circ w6[D6 + 1];

.var/dm x[5];        {dm(x) = input x, dm(x+i) = parallel outputs x_i}

i2 = ^w1;  L2 = %w1;
i3 = ^w2;  L3 = %w2;
i4 = ^w3;  L4 = %w3;
i5 = ^w4;  L5 = %w4;
i6 = ^w5;  L6 = %w5;
i7 = ^w6;  L7 = %w6;

zero(i2, m2, L2);
zero(i3, m3, L3);
zero(i4, m4, L4);
zero(i5, m5, L5);
zero(i6, m6, L6);
zero(i7, m7, L7);

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   si = mx1;                       {read current input into SI}
   sr = ashift si by -3 (hi);      {scale x down by a factor of 8}
   dm(x) = sr1;                    {save x}

{--- plain-1 -------------------------------------------------------------}

   mr = 0;
   mr1 = dm(x);                    {mr = x = input}

   mx1 = a1;
   tap(i2, m2, D1, my1);           {my1 = sD1 = D1-th tap}
   mr = mr + mx1 * my1 (rnd);      {mr = x1 = x + a1 * sD1}
   
   tapin(i2, m2, mr1);             {put x1 into tap-0}
   cdelay(i2, m2);                 {update delay}

   dm(x + 1) = mr1;                {save x1 for later usage}

{--- plain-2 -------------------------------------------------------------}

   mr = 0;
   mr1 = dm(x);                    {mr = x = input}

   mx1 = a2;
   tap(i3, m3, D2, my1);           {my1 = sD2 = D2-th tap}
   mr = mr + mx1 * my1 (rnd);      {mr = x2 = x + a2 * sD2}
   
   tapin(i3, m3, mr1);             {put x2 into tap-0}
   cdelay(i3, m3);                 {update delay}

   dm(x + 2) = mr1;                {save x2 for later usage}

{--- plain-3 -------------------------------------------------------------}

   mr = 0;
   mr1 = dm(x);                    {mr = x = input}

   mx1 = a3;
   tap(i4, m4, D3, my1);           {my1 = sD3 = D3-th tap}
   mr = mr + mx1 * my1 (rnd);      {mr = x3 = x + a3 * sD3}
   
   tapin(i4, m4, mr1);             {put x3 into tap-0}
   cdelay(i4, m4);                 {update delay}

   dm(x + 3) = mr1;                {save x3 for later usage}

{--- plain-4 -------------------------------------------------------------}

   mr = 0;
   mr1 = dm(x);                    {mr = x = input}

   mx1 = a4;
   tap(i5, m5, D4, my1);           {my1 = sD4 = D4-th tap}
   mr = mr + mx1 * my1 (rnd);      {mr = x4 = x + a4 * sD4}
   
   tapin(i5, m5, mr1);             {put x4 into tap-0}
   cdelay(i5, m5);                 {update delay}

   dm(x + 4) = mr1;                {save x4 for later usage}

{--- combine parallel outputs of plain sections --------------------------}

   mr = 0;
   mx0 = b1; my0 = dm(x+1);
   mr = mr + mx0 * my0 (ss);       {mr = b1 * x1}

   mx0 = b2; my0 = dm(x+2);
   mr = mr + mx0 * my0 (ss);       {mr = b1 * x1 + b2 * x2}

   mx0 = b3; my0 = dm(x+3);
   mr = mr + mx0 * my0 (ss);       {mr = b1 * x1 + b2 * x2 + b2 * x2}

   mx0 = b4; my0 = dm(x+4);
   mr = mr + mx0 * my0 (rnd);      {x5 = b1*x1 + b2*x2 + b2*x2 + b4*x4}
   if mv sat mr;                   {mr = x5}

{--- allpass-1 -----------------------------------------------------------}

   my0 = a5;
   tap(i6, m6, D5, mx0);           {mx0 = sD5 = D5-th tap}
   mr = mr + mx0 * my0 (rnd);      {mr = s05 = x5 + a5 * sD5}
   
   tapin(i6, m6, mr1);             {put s05 in tap-0 of delay-D5}
   cdelay(i6, m6);                 {update delay}

   mr =  mr1 * my0 (rnd);          {mr = a5 * s05}
   ay0 = mx0;                      {ay0 = sD5}
   ar =  ay0 - mr1;                {ar = x6 = sD5 - a5 * s05}

{--- allpass-2 -----------------------------------------------------------}

   mr = 0;
   mr1 = ar;                       {mr = x6 = input}

   my0 = a6;
   tap(i7, m7, D6, mx0);           {mx0 = sD6 = D6-th tap}
   mr = mr + mx0 * my0 (rnd);      {mr = s60 = x6 + a6 * sD6}
   
   tapin(i7, m7, mr1);             {put s06 in tap-0}
   cdelay(i7, m7);                 {update delay}

   mr =  mr1 * my0 (rnd);          {mr = a6 * s06}
   ay0 = mx0;                      {ay0 = sD6}
   ar =  ay0 - mr1;                {ar = y = sD6 - a6 * s06}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = ar;            {left output sample}
   dm(tx_buf + 2) = ar;            {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
