{canex1.dsp - canonical realization example using ccan}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Example 7.5.4 of Introduction to Signal Processing. The filter
 coefficients are:

       a = [1, 0, 0, -1]
       b = [1, 1, 2,  0] / 4 = [0.25, 0.25, 0.50, 0]

 The coefficients define a double-length concatenated circular buffer i7,
 and are stored in the files a.dec/a.hex and b.dec/b.hex. The input and 
 output signals are in the files xiir.dec/xiir.hex, yiir.dec/yiir.hex:

       x = 0.125 = 0x1000 = 1/8    y = 0.03125 = 0x0400 =  1/32
           0.375 = 0x3000 = 3/8        0.12500 = 0x1000 =  4/32
           0.250 = 0x2000 = 2/8        0.21875 = 0x1c00 =  7/32
           0.625 = 0x5000 = 5/8        0.43750 = 0x3800 = 14/32
           0.500 = 0x4000 = 4/8        0.53125 = 0x4400 = 17/32
           0.750 = 0x6000 = 6/8        0.84375 = 0x6c00 = 27/32
           0.000 = 0x0000 = 0/8        0.87500 = 0x7000 = 28/32
           0.000 = 0x0000 = 0/8        0.90625 = 0x7400 = 29/32
           0.000 = 0x0000 = 0/8        0.84375 = 0x6c00 = 27/32

 (The b-coefficients and x have been scaled down to fit in 1.15 format,
 as compared to Example 7.5.4.)

 Scaling factors:

    The a,b coefficients that are used in the program and read from
    the file ab.hex are the related to the true a,b coefficients by
    the following scale factors whoes exponents are passed to ccan():

       a_used = 2^(-ea) * a_true
       b_used = 2^(-eb) * b_true

    In this example: ea = 0, eb = 0, (no further scaling is needed)

    The input samples x must also, in general, be scaled down before
    passed into ccan() and then the output from ccan() must be scaled up:

       x1 = 2^(-ex) * x     input to ccan()
       y1 = ccan(x1)        output from ccan() due to x1
       y = 2^ex * y1        output due to x

    In this example, we must use ex = 1 (or larger).
    The unscaled input (ex = 0) causes overflows and the wrong output.
}

.include <c:\adi_dsp\macros\dspmac.dsp>;         {dsp macros}

.module/ram/abs=0  canex1;

.const        L=9;                        {input signal length}
.const        M=3;                        {filter order}
.var/dm       x[L];                       {input samples}
.var/dm       y[L];                       {output samples}
.var/dm/circ  w[M+1];                     {delay-line buffer in DM}
.var/pm/circ  a[M+1], b[M+1];             {concatenated coeffs in PM}

.init  x: <xiir.hex>;                     {input samples}
.init  a: <a.hex>;                        {denominator coefficients}
.init  b: <b.hex>;                        {numerator coefficients}

.const ea = 0;                            {scaling exponent for a}
.const eb = 0;                            {scaling exponent for b}
.const ex = 1;                            {pre/post scaling exponent}

        jump start; nop; nop; nop;               {Interupt vector table}
        rti; nop; nop; nop;                      {No interrupts used}
        rti; nop; nop; nop;
        rti; nop; nop; nop;
        rti; nop; nop; nop;
        rti; nop; nop; nop;
        rti; nop; nop; nop;

start:  i2 = ^w;  L2 = %w;                       {delay-line buffer}
        i4 = ^x;  m4 = 1;  L4 = 0;               {input samples}
        i5 = ^y;  m5 = 1;  L5 = 0;               {output samples}
        i7 = ^a;           L7 = 2*(M+1);         {double-length a,b coef-}
                                                 {-ficient buffer}
       zero(i2, m2, L2);                         {clear delay line}

       cntr = L;
       do outputs until ce;
              sr1 = dm(i4, m4);                         {read input x}
              sr = ashift sr1 by -ex (hi);              {pre-scale x down}
                                                        {compute output y}
              ccan(M, i7, m7, i2, m2, ea, eb, sr1);     {input from sr1}
                                                        {output in sr1}
              sr = ashift sr1 by ex (hi);               {post-scale y up}
outputs:      dm(i5, m5) = sr1;                         {write y}

        idle;

.endmod;
