{stereo.dsp - stereo delay effects with cross-feedback}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Fig.8.4.1 of Introduction to Signal Processing.
 Assuming the feedback filters are plain multipliers, GL(z)=aL,
 GR(z)=aR, the sample processing algorithm is:
    for each pair xL,xR do:
       sLL = tap(L, wL, pL, L)                   L-th tap of left delay
       sRR = tap(R, wR, pR, R)                   R-th tap of right delay
       yL = cL * xL + sLL                        left output
       yR = cR * xR + sRR                        right output
       *pL = bL * xL + aL * sLL + dR * sRR       input to left delay
       *pR = bR * xR + aR * sRR + dL * sLL       input to right delay
       cdelay(L, wL, &pL)                        update left delay
       cdelay(R, wR, &pR)                        update right delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const aL = 0x0000;                {aL = 0 - left self-feedback}
.const aR = 0x0000;                {aR = 0 - right self-feedback}

.const bL = 0x6666;                {bL = 0.8 - left delay gain}
.const bR = 0x6666;                {bR = 0.8 - right delay gain}

.const cL = 0x4000;                {cL = 0.5 - left direct path gain}
.const cR = 0x4000;                {cR = 0.5 - right direct path gain}

.const dL = 0x4000;                {dL = 0.5 - cross feedback from L to R}
.const dR = 0x4000;                {dR = 0.5 - cross feedback from R to L}

.const L = 3000;                   {TL = L/fs = 3/8 = 0.375 sec}
.const R = 3000;                   {TR = R/fs = 3/8 = 0.375 sec}

.var/dm/circ wL[L+1];              {left delay-line buffer}
.var/dm/circ wR[R+1];              {right delay-line buffer}

i2 = ^wL; L2 = %wL;                {delay-line buffer pointer and length}
i3 = ^wR; L3 = %wR;

zero(i2, m2, L2);                  {clear delay line}
zero(i3, m3, L3);

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   mx0 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm -----------------------------------------}

   tap(i2, m2, L, sr0);            {sr0 = sLL = L-th tap of left delay}
   tap(i3, m3, R, sr1);            {sr1 = sRR = R-th tap of right delay}

   mr = 0;
   mr1 = sr0;                      {mr1 = sLL}
   my0 = cL;
   mr = mr + mx0 * my0 (rnd);      {mr = yL = cL * xL + sLL = left output}
   if mv sat mr;

   dm(tx_buf + 1) = mr1;           {send left output to codec}

   mr = 0;
   mr1 = sr1;                      {mr1 = sRR}
   my0 = cR;
   mr = mr + mx1 * my0 (rnd);      {mr = yR = cR * xR + sRR = right output}
   if mv sat mr;

   dm(tx_buf + 2) = mr1;           {send right output to codec}

   my0 = bL;
   mr = mx0 * my0 (ss);            {mr = bL * xL}
   my0 = aL;
   mr = mr + sr0 * my0 (ss);       {mr = bL * xL + aL * sLL}
   my0 = dR;
   mr = mr + sr1 * my0 (rnd);      {mr = bL * xL + aL * sLL + dR * sRR}
   if mv sat mr;
   tapin(i2, m2, mr1);             {input to left delay}
   cdelay(i2, m2);                 {update left delay}

   my0 = bR;
   mr = mx1 * my0 (ss);            {mr = bR * xR}
   my0 = aR;
   mr = mr + sr1 * my0 (ss);       {mr = bR * xR + aR * sRR}
   my0 = dL;
   mr = mr + sr0 * my0 (rnd);      {mr = bR * xR + aR * sRR + dL * sLL}
   if mv sat mr;
   tapin(i3, m3, mr1);             {input to right delay}
   cdelay(i3, m3);                 {update right delay}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
